From 96daeb8d22e33e3484fefb27914bd434efd32142 Mon Sep 17 00:00:00 2001
From: Xi Shen <davidshen84@gmail.com>
Date: Wed, 8 Jun 2016 13:49:54 +0800
Subject: [PATCH] ob-sql.el: Support sqlcmd and cygwin environment

* lisp/ob-sql.el (org-babel-sql-dbstring-mssql): Format Microsoft
  sqlcmd command line args.

* lisp/ob-sql.el (platform-convert-file-name): Convert a *nix path to
  Windows long path in Cygwin environment, or do nothing.

* lisp/ob-sql.el (org-babel-execute:sql): Add `mssql` command support.
  For both `msosql` and `mssql` the `input` and `output` path will be
  mapped by `platform-convert-file-name`

The `osql` command line tool was last updated in 2004,
https://technet.microsoft.com/en-us/library/aa214012(v=sql.80).aspx,
and could not output the query result in a way that morden
`org-table.el` expects.  The `sqlcmd` is the preferred command line
tool to connect the Microsoft SQL Server and it also has a Linux
version,
https://msdn.microsoft.com/en-us/library/hh568447(v=sql.110).aspx.

TINYCHANGE
---
 lisp/ob-sql.el | 28 ++++++++++++++++++++++++++--
 1 file changed, 26 insertions(+), 2 deletions(-)

diff --git a/lisp/ob-sql.el b/lisp/ob-sql.el
index 6488afe..b115faa 100644
--- a/lisp/ob-sql.el
+++ b/lisp/ob-sql.el
@@ -103,6 +103,24 @@ Pass nil to omit that arg."
   "Make Oracle command line args for database connection."
   (format "%s/%s@%s:%s/%s" user password host port database))
 
+(defun org-babel-sql-dbstring-mssql (host user password database)
+  "Make Microsoft sqlcmd commmand line args for database
+connection."
+  (mapconcat 'identity
+	     (delq nil
+		   (list (when host (format "-S \"%s\"" host))
+			 (when user (format "-U \"%s\"" user))
+			 (when password (format "-P \"%s\"" password))
+			 (when database (format "-d \"%s\"" database))))
+	     " "))
+
+(defun platform-convert-file-name (file)
+  "In Cygwin environment convert the file path into Windows long
+path and quote it."
+  (if (fboundp 'cygwin-convert-file-name-to-windows)
+      (format "\"%s\"" (cygwin-convert-file-name-to-windows file))
+    file))
+
 (defun org-babel-execute:sql (body params)
   "Execute a block of Sql code with Babel.
 This function is called by `org-babel-execute-src-block'."
@@ -131,8 +149,14 @@ This function is called by `org-babel-execute-src-block'."
 				      (org-babel-process-file-name out-file)))
                     (`msosql (format "osql %s -s \"\t\" -i %s -o %s"
 				     (or cmdline "")
-				     (org-babel-process-file-name in-file)
-				     (org-babel-process-file-name out-file)))
+				     (platform-convert-file-name (org-babel-process-file-name in-file))
+				     (platform-convert-file-name (org-babel-process-file-name out-file))))
+		    (`mssql (format "sqlcmd %s -s \"\t\" %s -i %s -o %s"
+				     (or cmdline "")
+				     (org-babel-sql-dbstring-mssql
+				      dbhost dbuser dbpassword database)
+				     (platform-convert-file-name (org-babel-process-file-name in-file))
+				     (platform-convert-file-name (org-babel-process-file-name out-file))))
                     (`mysql (format "mysql %s %s %s < %s > %s"
 				    (org-babel-sql-dbstring-mysql
 				     dbhost dbport dbuser dbpassword database)
-- 
2.8.0

